/******/ (() => { // webpackBootstrap
/******/ 	"use strict";
/******/ 	var __webpack_modules__ = ({

/***/ "./libs/nodeplotlib/src/lib/interfaces/plot.ts":
/***/ ((__unused_webpack_module, exports) => {


Object.defineProperty(exports, "__esModule", ({ value: true }));


/***/ }),

/***/ "./libs/nodeplotlib/src/lib/nodeplotlib.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.plot = void 0;
const tslib_1 = __webpack_require__("tslib");
const core_1 = __webpack_require__("@nestjs/core");
const rxjs_1 = __webpack_require__("rxjs");
const plots_service_1 = __webpack_require__("./libs/nodeplotlib/src/lib/server/plots/plots.service.ts");
const server_module_1 = __webpack_require__("./libs/nodeplotlib/src/lib/server/server.module.ts");
const bridge_service_1 = __webpack_require__("./libs/nodeplotlib/src/lib/server/services/bridge.service.ts");
const get_port_1 = __webpack_require__("./libs/nodeplotlib/src/lib/utils/get-port.ts");
let app = null;
let plotsService;
let bridgeService;
// This variable is used to determine if the nestjs app is running
// or starting. Because it is "async" and the plot function is not,
// we need to make sure that we do not bootstrap the app twice in the
// same macro-task.
let appRuns = false;
let shutdownSubscription;
const plotsBuffer$ = new rxjs_1.BehaviorSubject([]);
const port = (0, get_port_1.getPort)();
/**
 * Plots the given data with the given layout. This function
 * starts a server if one is not already running.
 * @param data
 * @param layout
 * @param cb
 */
function plot(data, layout, config) {
    bootstrap(port);
    const bufferedPlots = plotsBuffer$.value;
    const streamData$ = data instanceof rxjs_1.Observable ? data : (0, rxjs_1.of)(data);
    plotsBuffer$.next([
        ...bufferedPlots,
        { data: streamData$, layout: (0, rxjs_1.of)(layout), config },
    ]);
}
exports.plot = plot;
function bootstrap(port) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (appRuns) {
            console.log('App is already up and running');
            return;
        }
        appRuns = true;
        app = yield core_1.NestFactory.create(server_module_1.ServerModule);
        plotsService = app.get(plots_service_1.PlotsService);
        bridgeService = app.get(bridge_service_1.BridgeService);
        yield app.listen(port);
        const actualPort = app.getHttpServer().address().port;
        bridgeService.setPort(actualPort);
        plotsService.setBuffer(plotsBuffer$);
        console.log('Server running at', `http://localhost:${actualPort}`);
        shutdownSubscription = bridgeService.shutdown$.subscribe(shutdown);
    });
}
function shutdown() {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        console.log('Server shutting down');
        shutdownSubscription === null || shutdownSubscription === void 0 ? void 0 : shutdownSubscription.unsubscribe();
        appRuns = false;
        if (app) {
            yield app.close();
        }
    });
}


/***/ }),

/***/ "./libs/nodeplotlib/src/lib/server/plots/plots.gateway.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


var _a, _b;
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.PlotsGateway = void 0;
const tslib_1 = __webpack_require__("tslib");
const websockets_1 = __webpack_require__("@nestjs/websockets");
const rxjs_1 = __webpack_require__("rxjs");
const operators_1 = __webpack_require__("rxjs/operators");
const bridge_service_1 = __webpack_require__("./libs/nodeplotlib/src/lib/server/services/bridge.service.ts");
const plots_service_1 = __webpack_require__("./libs/nodeplotlib/src/lib/server/plots/plots.service.ts");
let PlotsGateway = class PlotsGateway {
    constructor(bridgeService, plotsService) {
        this.bridgeService = bridgeService;
        this.plotsService = plotsService;
        this.clientMap = new Map();
        this.plotDataStream$ = this.plotsService.plotIds$.pipe((0, operators_1.switchMap)((plotIds) => (0, rxjs_1.merge)(...Array.from(plotIds).map((id) => {
            const plotDataStream = this.plotsService.plotEntities.get(id);
            if (!plotDataStream) {
                return new rxjs_1.Observable();
            }
            return (0, rxjs_1.combineLatest)([
                plotDataStream.data,
                plotDataStream.layout,
            ]).pipe((0, operators_1.map)(([data, layout]) => ({
                id,
                data,
                layout,
                config: plotDataStream.config,
            })));
        }))), (0, operators_1.share)());
    }
    handleEvent() {
        return this.plotDataStream$.pipe((0, operators_1.map)((plotData) => ({
            event: 'plotdata',
            data: plotData,
        })));
    }
    handleConnection(client) {
        console.log('client connected');
        this.clientMap.set(client, Date.now());
    }
    handleDisconnect(client) {
        console.log('client disconnected');
        this.clientMap.delete(client);
        if (this.clientMap.size === 0) {
            this.bridgeService.shutdown$.next(null);
        }
    }
};
tslib_1.__decorate([
    (0, websockets_1.SubscribeMessage)('readplots'),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", []),
    tslib_1.__metadata("design:returntype", void 0)
], PlotsGateway.prototype, "handleEvent", null);
PlotsGateway = tslib_1.__decorate([
    (0, websockets_1.WebSocketGateway)({ transports: ['polling'] }),
    tslib_1.__metadata("design:paramtypes", [typeof (_a = typeof bridge_service_1.BridgeService !== "undefined" && bridge_service_1.BridgeService) === "function" ? _a : Object, typeof (_b = typeof plots_service_1.PlotsService !== "undefined" && plots_service_1.PlotsService) === "function" ? _b : Object])
], PlotsGateway);
exports.PlotsGateway = PlotsGateway;


/***/ }),

/***/ "./libs/nodeplotlib/src/lib/server/plots/plots.service.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.PlotsService = void 0;
const tslib_1 = __webpack_require__("tslib");
const common_1 = __webpack_require__("@nestjs/common");
const rxjs_1 = __webpack_require__("rxjs");
const operators_1 = __webpack_require__("rxjs/operators");
let PlotsService = class PlotsService {
    constructor() {
        this.plotEntities = new Map();
        this.plotIds$ = new rxjs_1.BehaviorSubject(new Set());
        this.currentPlotId = 0;
    }
    setBuffer(buffer$) {
        var _a;
        (_a = this.bufferSubscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        this.bufferSubscription = buffer$
            .pipe((0, operators_1.filter)((buffer) => buffer.length > 0))
            .subscribe((buffer) => this.readBuffer(buffer));
    }
    addPlot(plotData) {
        const plot = {
            id: this.currentPlotId++,
            data: plotData.data,
            layout: plotData.layout,
            config: plotData.config,
        };
        this.plotEntities.set(plot.id, plot);
        const plotIds = this.plotIds$.value;
        plotIds.add(plot.id);
        this.plotIds$.next(plotIds);
    }
    /**
     * Function gets executed on the main process and makes the service read
     * the buffered plot data.
     * @param buffer
     */
    readBuffer(buffer) {
        for (const plot of buffer) {
            this.addPlot(plot);
        }
    }
};
PlotsService = tslib_1.__decorate([
    (0, common_1.Injectable)()
], PlotsService);
exports.PlotsService = PlotsService;


/***/ }),

/***/ "./libs/nodeplotlib/src/lib/server/server.module.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ServerModule = void 0;
const tslib_1 = __webpack_require__("tslib");
const common_1 = __webpack_require__("@nestjs/common");
const serve_static_1 = __webpack_require__("@nestjs/serve-static");
const path_1 = __webpack_require__("path");
const plots_gateway_1 = __webpack_require__("./libs/nodeplotlib/src/lib/server/plots/plots.gateway.ts");
const plots_service_1 = __webpack_require__("./libs/nodeplotlib/src/lib/server/plots/plots.service.ts");
const bridge_service_1 = __webpack_require__("./libs/nodeplotlib/src/lib/server/services/bridge.service.ts");
let ServerModule = class ServerModule {
};
ServerModule = tslib_1.__decorate([
    (0, common_1.Module)({
        imports: [
            serve_static_1.ServeStaticModule.forRoot({
                rootPath: (0, path_1.join)(__dirname, 'web'),
            }),
        ],
        providers: [plots_gateway_1.PlotsGateway, plots_service_1.PlotsService, bridge_service_1.BridgeService],
    })
], ServerModule);
exports.ServerModule = ServerModule;


/***/ }),

/***/ "./libs/nodeplotlib/src/lib/server/services/bridge.service.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.BridgeService = void 0;
const tslib_1 = __webpack_require__("tslib");
const common_1 = __webpack_require__("@nestjs/common");
const rxjs_1 = __webpack_require__("rxjs");
const open_window_1 = __webpack_require__("./libs/nodeplotlib/src/lib/utils/open-window.ts");
/**
 * The BridgeService is used to build a bridge between the
 * NestJS server and the outside of the NestJS server.
 * The two purposes are to get the port of the express server
 * inside of the NestJS server and to provide a shutdown$
 * steam so that the NestJS server can trigger a full shutdown.
 *
 * @see nodeplotlib.ts
 */
let BridgeService = class BridgeService {
    constructor() {
        this.shutdown$ = new rxjs_1.Subject();
        this.port$ = new rxjs_1.Subject();
        this.port$.subscribe((port) => {
            (0, open_window_1.openWindow)(`http://localhost:${port}`);
        });
    }
    setPort(port) {
        this.port$.next(port);
    }
    shutdown() {
        this.shutdown$.next(null);
    }
};
BridgeService = tslib_1.__decorate([
    (0, common_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], BridgeService);
exports.BridgeService = BridgeService;


/***/ }),

/***/ "./libs/nodeplotlib/src/lib/utils/get-port.ts":
/***/ ((__unused_webpack_module, exports) => {


Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.getPort = void 0;
function getPort() {
    const portAsString = process.env.NODEPLOTLIB_PORT;
    const port = Number(portAsString);
    if (isNaN(port)) {
        return 0;
    }
    return port;
}
exports.getPort = getPort;


/***/ }),

/***/ "./libs/nodeplotlib/src/lib/utils/open-window.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.openWindow = void 0;
const child_process_1 = __webpack_require__("child_process");
const os_1 = __webpack_require__("os");
function openWindow(location) {
    if (process.env.NODEPLOTLIB_PORT) {
        return;
    }
    switch ((0, os_1.type)()) {
        case 'Linux':
            (0, child_process_1.exec)(`xdg-open ${location}`);
            break;
        case 'Darwin':
            (0, child_process_1.exec)(`open ${location}`);
            break;
        case 'Windows_NT':
            (0, child_process_1.exec)(`start ${location}`);
            break;
    }
}
exports.openWindow = openWindow;


/***/ }),

/***/ "@nestjs/common":
/***/ ((module) => {

module.exports = require("@nestjs/common");

/***/ }),

/***/ "@nestjs/core":
/***/ ((module) => {

module.exports = require("@nestjs/core");

/***/ }),

/***/ "@nestjs/serve-static":
/***/ ((module) => {

module.exports = require("@nestjs/serve-static");

/***/ }),

/***/ "@nestjs/websockets":
/***/ ((module) => {

module.exports = require("@nestjs/websockets");

/***/ }),

/***/ "rxjs":
/***/ ((module) => {

module.exports = require("rxjs");

/***/ }),

/***/ "rxjs/operators":
/***/ ((module) => {

module.exports = require("rxjs/operators");

/***/ }),

/***/ "tslib":
/***/ ((module) => {

module.exports = require("tslib");

/***/ }),

/***/ "child_process":
/***/ ((module) => {

module.exports = require("child_process");

/***/ }),

/***/ "os":
/***/ ((module) => {

module.exports = require("os");

/***/ }),

/***/ "path":
/***/ ((module) => {

module.exports = require("path");

/***/ })

/******/ 	});
/************************************************************************/
/******/ 	// The module cache
/******/ 	var __webpack_module_cache__ = {};
/******/ 	
/******/ 	// The require function
/******/ 	function __webpack_require__(moduleId) {
/******/ 		// Check if module is in cache
/******/ 		var cachedModule = __webpack_module_cache__[moduleId];
/******/ 		if (cachedModule !== undefined) {
/******/ 			return cachedModule.exports;
/******/ 		}
/******/ 		// Create a new module (and put it into the cache)
/******/ 		var module = __webpack_module_cache__[moduleId] = {
/******/ 			// no module.id needed
/******/ 			// no module.loaded needed
/******/ 			exports: {}
/******/ 		};
/******/ 	
/******/ 		// Execute the module function
/******/ 		__webpack_modules__[moduleId](module, module.exports, __webpack_require__);
/******/ 	
/******/ 		// Return the exports of the module
/******/ 		return module.exports;
/******/ 	}
/******/ 	
/************************************************************************/
var __webpack_exports__ = {};
// This entry need to be wrapped in an IIFE because it need to be isolated against other modules in the chunk.
(() => {
var exports = __webpack_exports__;

Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.PlotData = exports.plot = exports.Config = exports.Layout = exports.Plot = void 0;
// Exports required for the actual nodeplotlib
var plot_1 = __webpack_require__("./libs/nodeplotlib/src/lib/interfaces/plot.ts");
Object.defineProperty(exports, "Plot", ({ enumerable: true, get: function () { return plot_1.Plot; } }));
Object.defineProperty(exports, "Layout", ({ enumerable: true, get: function () { return plot_1.Layout; } }));
Object.defineProperty(exports, "Config", ({ enumerable: true, get: function () { return plot_1.Config; } }));
var nodeplotlib_1 = __webpack_require__("./libs/nodeplotlib/src/lib/nodeplotlib.ts");
Object.defineProperty(exports, "plot", ({ enumerable: true, get: function () { return nodeplotlib_1.plot; } }));
// Exports required for the web app
var plot_2 = __webpack_require__("./libs/nodeplotlib/src/lib/interfaces/plot.ts");
Object.defineProperty(exports, "PlotData", ({ enumerable: true, get: function () { return plot_2.PlotData; } }));

})();

var __webpack_export_target__ = exports;
for(var i in __webpack_exports__) __webpack_export_target__[i] = __webpack_exports__[i];
if(__webpack_exports__.__esModule) Object.defineProperty(__webpack_export_target__, "__esModule", { value: true });
/******/ })()
;
//# sourceMappingURL=main.js.map